## http://github.com/boutiques/boutiques-schema
(this documentation was generated by [jsonschema-md](https://www.npmjs.com/package/jsonschema-md))

#### Type: object

| Name    | Type    | Description | Example |
| ------- | ------- | ----------- | ------- |
| name | string | Tool name. |  |
| tool-version | string | Tool version. |  |
| description | string | Tool description. |  |
| command-line | string | A string that describes the tool command line, where input and output values are identified by "keys". At runtime, command-line keys are substituted with flags and values. |  |
| [container-image](#container-image) | object |  |  |
| schema-version | string | Version of the schema used. |  |
| environment-variables | array | An array of key-value pairs specifying environment variable names and their values to be used in the execution environment. |  |
| groups | array | Sets of identifiers of inputs, each specifying an input group. |  |
| inputs | array |  |  |
| output-files | array |  |  |
| [invocation-schema](#invocation-schema) | object |  |  |
| [suggested-resources](#suggested-resources) | object |  |  |
| [custom](#custom) | object |  |  |
Allowed schema-version: ENUM(0.5)


*****

## container-image

#### Type: object

*****

## environment-variables

#### Type: object

Required: name, value

| Name    | Type    | Description | Example |
| ------- | ------- | ----------- | ------- |
| name | string | The environment variable name (identifier) containing only alphanumeric characters and underscores. Example: "PROGRAM_PATH". |  |
| value | string | The value of the environment variable. |  |
| description | string | Description of the environment variable. |  |
Allowed name: PATTERN(^[a-z,A-Z][0-9,_,a-z,A-Z]*$)


*****

## groups

#### Type: object

Required: name, id, members

| Name    | Type    | Description | Example |
| ------- | ------- | ----------- | ------- |
| id | string | A short, unique, informative identifier containing only alphanumeric characters and underscores. Typically used to generate variable names. Example: "outfile_group". |  |
| name | string | A human-readable name for the input group. |  |
| description | string | Description of the input group. |  |
| members | array | IDs of the inputs belonging to this group. |  |
| mutually-exclusive | boolean | True if only one input in the group may be active at runtime. |  |
| one-is-required | boolean | True if at least one of the inputs in the group must be active at runtime. |  |
| all-or-none | boolean | True if members of the group need to be toggled together |  |
Allowed id: PATTERN(^[0-9,_,a-z,A-Z]*$)

Allowed members: PATTERN(^[0-9,_,a-z,A-Z]*$)


*****

## members

#### Type: string

Allowed: PATTERN(^[0-9,_,a-z,A-Z]*$)


*****

## inputs

#### Type: object

Required: name, id, type

| Name    | Type    | Description | Example |
| ------- | ------- | ----------- | ------- |
| id | string | A short, unique, informative identifier containing only alphanumeric characters and underscores. Typically used to generate variable names. Example: "data_file". |  |
| name | string | A human-readable input name. Example: 'Data file'. |  |
| type | string | Input type. |  |
| description | string | Input description. |  |
| value-key | string | A string contained in command-line, substituted by the input value and/or flag at runtime. |  |
| list | boolean | True if input is a list of value. An input of type "Flag" cannot be a list. |  |
| optional | boolean | True if input is optional. |  |
| command-line-flag | string | Option flag of the input, involved in the value-key substitution. Inputs of type "Flag" have to have a command-line flag. Examples: -v, --force. |  |
| requires-inputs | array | Ids of the inputs which must be active for this input to be available. |  |
| disables-inputs | array | Ids of the inputs that are disabled when this input is active. |  |
| command-line-flag-separator | string | Separator used between flags and their arguments. Defaults to a single space. |  |
| default-value | undefined | Default value of the input, used by the tool when no option is specified. |  |
| value-choices | array | Permitted choices for input value. May not be used with the Flag type. |  |
| integer | boolean | Specify whether the input should be an integer. May only be used with Number type inputs. |  |
| minimum | number | Specify the minimum value of the input (inclusive). May only be used with Number type inputs. |  |
| maximum | number | Specify the maximum value of the input (inclusive). May only be used with Number type inputs. |  |
| exclusive-minimum | boolean | Specify whether the minimum is exclusive or not. May only be used with Number type inputs. |  |
| exclusive-maximum | boolean | Specify whether the maximum is exclusive or not. May only be used with Number type inputs. |  |
| min-list-entries | number | Specify the minimum number of entries in the list. May only be used with List type inputs. |  |
| max-list-entries | number | Specify the maximum number of entries in the list. May only be used with List type inputs. |  |
| uses-absolute-path | boolean | Specifies that this input must be given as an absolute path. Only specifiable for File type inputs. |  |
Allowed id: PATTERN(^[0-9,_,a-z,A-Z]*$)

Allowed type: ENUM(String, File, Flag, Number)


*****

## requires-inputs

#### Type: string


*****

## disables-inputs

#### Type: string

*****

## value-choices

#### Type: undefined

*****

## output-files

#### Type: object

Required: id, name, path-template

| Name    | Type    | Description | Example |
| ------- | ------- | ----------- | ------- |
| id | string | A short, unique, informative identifier containing only alphanumeric characters and underscores. Typically used to generate variable names. Example: "data_file" |  |
| name | string | A human-readable output name. Example: 'Data file' |  |
| description | string | Output description. |  |
| value-key | string | A string contained in command-line, substituted by the output value and/or flag at runtime. |  |
| path-template | string | Describes the output file path relatively to the execution directory. May contain input value keys. Example: "results/[INPUT1]_brain.mnc". |  |
| path-template-stripped-extensions | array | List of file extensions that will be stripped from the input values before being substituted in the path template. Example: [".nii",".nii.gz"]. |  |
| list | boolean | True if output is a list of value. |  |
| optional | boolean | True if output may not be produced by the tool. |  |
| command-line-flag | string | Option flag of the output, involved in the value-key substitution. Examples: -o, --output |  |
| command-line-flag-separator | string | Separator used between flags and their arguments. Defaults to a single space. |  |
| uses-absolute-path | boolean | Specifies that this output filepath will be given as an absolute path. |  |
| file-template | array | An array of strings that may contain value keys. Each item will be a line in the configuration file. |  |
Allowed id: PATTERN(^[0-9,_,a-z,A-Z]*$)


*****

## path-template-stripped-extensions

#### Type: string


*****

## file-template

#### Type: string


*****

## invocation-schema

#### Type: object


*****

## suggested-resources

#### Type: object

| Name    | Type    | Description | Example |
| ------- | ------- | ----------- | ------- |
| cpu-cores | integer | The requested number of cpu cores to run the described application |  |
| ram | number | The requested number of GB RAM to run the described application |  |
| disk-space | number | The requested number of GB of storage to run the described application |  |
| nodes | integer | The requested number of nodes to spread the described application across |  |
| walltime-estimate | number | Estimated wall time of a task in seconds. |  |

*****

## custom

#### Type: object

*****



## Command-line substitution

At runtime, a __value__ is assigned to each input in ```inputs```.

* Inputs of type "String" may contain any string (encoding is not specified).
* Inputs of type "Number" must contain a string representing a number.
* Inputs of type "File" must contain a string representing a file path (absolute or relative to the execution directory).
* Inputs of type "Flag" must contain a boolean.

When input is a list, __value__ contains the concatenation of all strings in the list, separated by spaces. Spaces included in list elements must be escaped by '\', or the elements must be single-quoted (Linux conventions).  An input of type "Flag" may not be a list.

The tool command line is generated as follows:

1. For each input in ```inputs```, where input has a ```value-key``` and input has __value__:
  * In ```command-line```, replace input ```value-key``` by:
         * ```command-line-flag``` if input is of type "Flag".
         * ```command-line-flag``` ```command-line-flag-separator``` __value__ (i.e. separated only by ```command-line-flag-separator``` if it is present).
         * ```command-line-flag``` __value__ (space-separated) otherwise.
  * For each output in ```output-files```
         * If ```path-template``` contains input ```value-key```, replace ```value-key``` by __value__, having previously removed the last occurrence of character '.' and subsequent characters from __value__ when input is of type "File".
2. For each output in ```output-files```, where output has a ```value-key```:
  * In ```command-line```, replace output ```value-key``` by:
         * ```command-line-flag``` ```path-template``` (space-separated). At this step, input ```value-key```s contained in ```path-template``` are already substituted.

## Examples

An annotated example of a Boutiques application descriptor:
```javascript
{
  // An example Boutiques JSON descriptor
  // Note that real JSON cannot include comments like this
  "name" : "Example Boutiques Tool",
  "tool-version" : "0.0.1",
  "description" : "This property describes the tool or application",
  // The actual command line run will be generated by substituting given input values into this string,
  // using the "value-key" property of the input. Notice we can chain multiple commands easily.
  "command-line" : "exampleTool_1 [STRING_INPUT] [FILE_INPUT] [ENUM_INPUT] | exampleTool_2 [FLAG_INPUT] [NUMBER_INPUT] >> [LOG]",
  // Optionally specify a container image that can be used to execute the tool
  "container-image": {
        "type": "docker",
        "image": "boutiques/examples"
  },
  "schema-version" : 0.5, // The version of the Boutiques schema this descriptor follows
  "inputs" : [{
    // A string parameter
    "id" : "str_input", // A unique id for this input (e.g. for platforms to differentiate inputs)
    "name" : "A string input", // A human readable name for the input parameter (e.g. for UI display)
    "type" : "String",
    "description" : "Describe the use and meaning of the parameter",
    "optional" : false, // Means the parameter is required for the tool
    // This will be prepended to the input value before substitution into the command line
    "command-line-flag" : "-i",
    "value-key" : "[STRING_INPUT]", // Used for substitution into the command-line above
    "list" : true, // When true, allows multiple inputs to be given for this parameter
    // Restrict lists to have only a specific number of members
    "min-list-entries" : 1,
    "max-list-entries" : 3,
    // Default values can also be specified for parameters
    "default-value" : "aStringValue"
  }, {
    // A numerical parameter
    "id" : "num_input",
    "name" : "A number input",
    "type" : "Number",
    "value-key" : "[NUMBER_INPUT]",
    "optional" : true, // Means this parameter is not strictly required for the tool to be used
    // Can alter the prefix flag to use a different separator (e.g. here -n=<inputval>)
    "command-line-flag" : "-n",
    "command-line-separator" : "=",
    // Use constraints to specify a number in (0,1]
    "minimum" : 0,
    "maximum" : 1,
    "exclusive-minimum" : true,
    "exclusive-maximum" : false,
    "integer" : false // Make 'true' to require the number is an integer
  }, {
    // File input parameters are specified like strings
    // Implementation details, like validation of file existence, are left to platforms
    "id" : "file_input",
    "name" : "A file input",
    "type" : "File",
    "optional" : true,
    "value-key" : "[FILE_INPUT]"
  }, {
    // An enum-type input
    "id" : "enum_input",
    "name" : "An enum input",
    "type" : "Enum",
    "enum-value-choices" : ["val1","val2","val3"],
    "value-key" : "[ENUM_INPUT]"
  }, {
    // A flag input: essentially a boolean describing the presence or absence of a command-line flag
    "id" : "flag_input",
    "name" : "A flag input",
    "type" : "Flag",
    // Command line substitution can only be the flag or nothing (no extra values given)
    "command-line-flag" : "-f",
    "value-key" : "[FLAG_INPUT]",
    // Can specify a list of ids of parameters that this input requires to be used
    "requires-inputs" : ["file_input"],
    // Can also specify a list of ids that are not permitted to be used (added to command-line)
    // when this parameter is active (given a value)
    "disables-inputs" : ["num_input"]
  }],
  // This section specifies the set of output files from the tool
  "output-files" : [{
    "id" : "logfile", // As above, a unique id for the output
    "name" : "Log file",
    "description" : "The output log file from the example tool",
    // Path-template specifies where the output file will be
    // Note that it can automatically depend on the input values
    "path-template" : "log-[STRING_INPUT].txt",
    // This specifies the substitution key of path-template into the command-line
    "value-key" : "[LOG]",
    // Any input keys (e.g. here str_input) will have any of the following extensions removed
    // from their value before substitution (e.g. if 'in.csv' was given to str_input, 'in' would
    // would be the resulting substituted value in [STRING_INPUT])
    "path-template-stripped-extensions" : [".txt", ".csv"],
    // As above, optional : false requires that the output file exists
    "optional" : false
  }, {
    // We can also represent sets of output files using a wildcard
    "id" : "output_files",
    "name" : "outfiles",
    // When capturing multiple files, list should be true
    "list" : true,
    // A regular expression style wildcard (*) is used to capture multiple files
    "path-template" : "output/*_exampleOutputTag.resultType",
    "optional" : true // I.e. this output may not necessarily be created
  }],
  // Parameter groups represent semantically related sets of input parameters. They can be used
  // to specify inter-parameter disabling/requirement, and to help automatic UI generation on platforms.
  "groups" : [{
    "id" : "an_example_group",
    "name" : "Example group 1",
    "description" : "This group forces us to choose exactly one of the file or the enum input to use.",
    // The set of ids that form the member parameters of the group
    "members" : ["file_input", "enum_input"],
    // When true, only one of the group members can be active (given as input) at a time
    "mutually-exclusive" : true,
    // When true, at least one of the group members must be specified in the input values
    "one-is-required" : true
  }]
}
```

See the [cbrain-plugins-neuro repo](https://github.com/aces/cbrain-plugins-neuro/tree/master/cbrain_task_descriptors "Boutiques example descriptors at cbrain-plugins-neuro") for some example descriptors currently in use.

More examples (TODO)
* Inputs without value-key.
* List outputs without a value-key.

